/*
 * pdscoin.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#include <stdlib.h>

#include <pds/pdscongruential.h>
#include <pds/pdscoin.h>
#include <pds/pdsvector.h>

////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsCoin                                                ////
////////////////////////////////////////////////////////////////////////////////


/** \fn PdsCoin *pds_coin_new(PdsRvReal p)
 *  \brief Crea una variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad p de obtener un 1 al ser jugada.
 *  \param[in] p Probabilidad de la moneda de obtener un 1.
 *  \return Un puntero a la variable aleatoria de tipo PdsCoin.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsCoinGroup
 */
PdsCoin *pds_coin_new(PdsRvReal p)
{
	PdsCoin *X=NULL;

	if(p>1.0)	return NULL;
	if(p<0.0)	return NULL;

	X=(PdsCoin *)calloc(1,sizeof(PdsCoin));
	if(X==NULL)	return NULL;

	X->X1=pds_congruential_new();
	X->p=p;

	if((X->X1->xn*1.0/PDS_RAND_MAX)<p)	X->x=+1;
	else					X->x=-1;

	return X;
}

/** \fn int pds_coin_init(PdsCoin *X, PdsRvReal x0)
 *  \brief Inicia con un valor x0 {-1,+1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[in] x0 Valor inicial de la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_init(PdsCoin *X, PdsRvReal x0)
{
	if(X==NULL)	return FALSE;

	if(x0<0)
	{
		X->x=-1;
		pds_congruential_init(X->X1,PDS_RAND_MAX-1);

	}
	else
	{
		X->x=+1;	
		pds_congruential_init(X->X1,0);
	}

	return TRUE;
}

/** \fn int pds_coin_set_p(PdsCoin *X, PdsRvReal p)
 *  \brief Inicia con una probabilidad de p a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[in] p Valor de probabilidad de la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_set_p(PdsCoin *X, PdsRvReal p)
{
	if(X==NULL)	return FALSE;
	X->p=p;
	return TRUE;
}

/** \fn int pds_coin_get_value(PdsCoin *X, PdsRvReal *x)
 *  \brief Pide un valor {-1,1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Valor de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_value(PdsCoin *X, PdsRvReal *x)
{
	PdsRvNaturalD xn;

	if(X==NULL)	return FALSE;
	if(x==NULL)	return FALSE;

	pds_congruential_get_value(X->X1,&xn);

	if( (xn*1.0/PDS_RAND_MAX)<X->p )	X->x=+1;
	else					X->x=-1;

	*x=X->x;

	return TRUE;
}

/** \fn int pds_coin_get_bit(PdsCoin *X, PdsRvByte *x)
 *  \brief Pide un valor {0,1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Valor de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_bit(PdsCoin *X, PdsRvByte *x)
{
	PdsRvNaturalD xn;

	if(X==NULL)	return FALSE;
	if(x==NULL)	return FALSE;

	pds_congruential_get_value(X->X1,&xn);

	if( (xn*1.0/PDS_RAND_MAX)<X->p )	X->x=+1;
	else					X->x=-1;

	*x=(PdsRvByte)((X->x+1)/2);

	return TRUE;
}

/** \fn int pds_coin_get_last_value(PdsCoin *X, PdsRvReal *x)
 *  \brief Pide el último valor {-1,1} obtenido de la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Ultimo valor obtenido de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_last_value(PdsCoin *X, PdsRvReal *x)
{
	if(X==NULL)	return FALSE;
	if(x==NULL)	return FALSE;

	*x=X->x;
	return TRUE;
}

/** \fn int pds_coin_get_last_bit(PdsCoin *X, PdsRvByte *x)
 *  \brief Pide el último valor {0,1} obtenido de la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Ultimo valor obtenido de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_last_bit(PdsCoin *X, PdsRvByte *x)
{
	if(X==NULL)	return FALSE;
	if(x==NULL)	return FALSE;

	*x=(PdsRvByte)((X->x+1)/2);
	return TRUE;
}

/** \fn int pds_coin_get_vector(PdsCoin *X, PdsVector *V)
 *  \brief Pide un vector de valores {-1,+1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] V Vector a cargar con valores de la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL o V==NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_vector(PdsCoin *X, PdsVector *V)
{
	PdsRvNaturalD xn;
	PdsRaNatural i;

	if(X==NULL)	return FALSE;
	if(V==NULL)	return FALSE;

	for(i=0;i<V->Nel;i++)
	{
		pds_congruential_get_value(X->X1,&xn);
	
		if( (xn*1.0/PDS_RAND_MAX)<X->p )	X->x=+1;
		else					X->x=-1;

		V->V[i]=X->x;
	}

	return TRUE;
}

/** \fn void pds_coin_free(PdsCoin *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsCoin.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCoinGroup
 */
void pds_coin_free(PdsCoin *X)
{
	if(X!=NULL)
	{
		pds_congruential_free(X->X1);
		free(X);
	}
}

/** \fn void pds_coin_destroy(PdsCoin **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsCoin, y la carga con NULL.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCoinGroup
 */
void pds_coin_destroy(PdsCoin **X)
{
	if((*X)!=NULL)
	{
		pds_congruential_free((*X)->X1);
		free((*X));
		(*X)=NULL;
	}

}


