/*
 * pdsrvcongruential.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#include <time.h>
#include <stdlib.h>
#include <math.h>

#include <pds/pdscongruential.h>


////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsCongruential                                        ////
////////////////////////////////////////////////////////////////////////////////


// Evalua el Incremento.
double pds_inc(	unsigned long int t3,unsigned long int t5,
			unsigned long int t7,unsigned long int t11,
			unsigned long int t13,unsigned long int t17)
{
	return pow(3.0,t3)*pow(5.0,t5)*pow(7.0,t7)*pow(11.0,t11)*pow(13.0,t13)*pow(17.0,t17);
}

// Encuentra los indices.
void pds_next_inc(	unsigned long int *t3 ,unsigned long int *t5,
			unsigned long int *t7 ,unsigned long int *t11,
			unsigned long int *t13,unsigned long int *t17)
{

	if(pds_inc((*t3)+1,*t5,*t7,*t11,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t3=(*t3)+1;}
	else
	{	*t3=1; 
		if(pds_inc(*t3,(*t5)+1,*t7,*t11,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t5=(*t5)+1;}
		else
		{	*t5=0;
			if(pds_inc(*t3,*t5,(*t7)+1,*t11,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t7=(*t7)+1;}
			else
			{	*t7=0;
				if(pds_inc(*t3,*t5,*t7,(*t11)+1,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t11=(*t11)+1;}
				else
				{	*t11=0;
					if(pds_inc(*t3,*t5,*t7,*t11,(*t13)+1,*t17)<(PDS_RAND_MAX*1.0))	{*t13=(*t13)+1;}
					else
					{	*t13=0;
						if(pds_inc(*t3,*t5,*t7,*t11,*t13,(*t17)+1)<(PDS_RAND_MAX*1.0))	{*t17=(*t17)+1;}
						else	{*t17=0;*t3=1;*t5=0;*t7=0;*t11=0;*t13=0;}
					}
				}
			}
		}		
	}

}

/** \fn PdsCongruential *pds_congruential_new(void)
 *  \brief Crea una secuencia aleatoria de tipo PdsCongruential.
 *   
 *  cada vez que se invoca a esta funcion se crea una nueva variable aleatoria
 *  con un desfase en amplitud distinto, cambiando (c) de [a*x_n+c]%PDS_RAND_MAX.
 *  No solo eso cada vez que se crea una nueva variable, el valor de inicio temporal
 *  x_n cambia usando la funcion time() y clock().
 *  \return Un puntero a la estructura de la secuencia aleatoria de tipo PdsCongruential.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsCongruentialGroup
 */
PdsCongruential *pds_congruential_new(void)
{
	PdsCongruential *X=NULL;

	static unsigned long int t3=1,t5=0,t7=0,t11=0,t13=0,t17=0;
	PdsRvReal C;

	X=(PdsCongruential *)calloc(1,sizeof(PdsCongruential));
	if(X==NULL) { return NULL;}


	X->xn=(time(NULL)%(PDS_RAND_MAX/2))+1+(clock()%(PDS_RAND_MAX/8)) ;
	X->x0=X->xn;
	X->a=1103515245;

	C=pds_inc(t3,t5,t7,t11,t13,t17);
	if(C<(PDS_RAND_MAX*1.0))	{X->c=(unsigned long int)(C);}
	else				{ return NULL;}

	// Buscando el nuevo incremento de c.
	pds_next_inc(&t3,&t5,&t7,&t11,&t13,&t17);

	return X;
}

/** \fn int pds_congruential_init(PdsCongruential *X,PdsRvNaturalD x0)
 *  \brief Inicializa la secuencia aleatoria de tipo PdsCongruential con x0.
 *  \param[in] X Variable secuencia aleatoria congruential.
 *  \param[in] x0 X{inicial}=x0 \% PDS_RAND_MAX.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsCongruentialGroup
 */
int pds_congruential_init(PdsCongruential *X,PdsRvNaturalD x0)
{
	if(X==NULL)	return FALSE;
	X->xn=x0%PDS_RAND_MAX;
	X->x0=X->xn;
	return TRUE;
}

/** \fn int pds_congruential_get_value(PdsCongruential *X,PdsRvNaturalD *x)
 *  \brief Retorna un número de la secuencia pseudo aleatorio entre [0,PDS_RAND_MAX).
 *  \param[in] X Variable secuencia aleatoria congruential.
 *  \param[out] x x es cargado con un número entre [0,PDS_RAND_MAX). Incluye 0 pero no  PDS_RAND_MAX.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsCongruentialGroup
 */
int pds_congruential_get_value(PdsCongruential *X,PdsRvNaturalD *x)
{
	if(X==NULL)	{*x=0;	return FALSE;}
	X->x0=X->xn;
	X->xn=(X->a*X->x0+X->c)%PDS_RAND_MAX;
	
	*x=X->x0;

	return TRUE;
}


/** \fn int pds_congruential_get_last_value(PdsCongruential *X,PdsRvNaturalD *x)
 *  \brief Retorna el último número entregado por la secuencia pseudo aleatorio entre [0,PDS_RAND_MAX).
 *  \param[in] X Variable secuencia aleatoria congruential.
 *  \param[out] x x es cargado con un número entre [0,PDS_RAND_MAX). Incluye 0 pero no  PDS_RAND_MAX.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsCongruentialGroup
 */
int pds_congruential_get_last_value(PdsCongruential *X,PdsRvNaturalD *x)
{
	if(X==NULL)	{*x=0;	return FALSE;}
	
	*x=X->x0;

	return TRUE;
}

/** \fn void pds_congruential_free(PdsCongruential *X)
 *  \brief Libera la variable secuencia aleatoria de tipo puntero PdsCongruential.
 *  \param[in] X La variable secuencia aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCongruentialGroup
 */
void pds_congruential_free(PdsCongruential *X)
{
	if(X!=NULL)
	{
		free(X);
	}
}


/** \fn void pds_congruential_destroy(PdsCongruential **X)
 *  \brief Libera la variable secuencia aleatoria de tipo puntero PdsCongruential, y la carga con NULL.
 *  \param[in] X La variable secuencia aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCongruentialGroup
 */
void pds_congruential_destroy(PdsCongruential **X)
{
	if((*X)!=NULL)
	{
		free(*X);
		(*X)=NULL;
	}
}


