/*
 * pdsexponential.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#include <stdlib.h>
#include <math.h>

#include <pds/pdscongruential.h>
#include <pds/pdsexponential.h>
#include <pds/pdsvector.h>


////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsExponential                                       ////
////////////////////////////////////////////////////////////////////////////////



/** \fn PdsExponential *pds_exponential_new(PdsRvReal Lambda)
 *  \brief Crea una variable aleatoria de tipo PdsExponential.
 *  \param[in] Lambda La inversa de la media de la variable aleatoria exponencial.
 *		si Lambda es menor igual a cero se retorna NULL.
 *  \return Un puntero a la variable aleatoria de tipo Exponential,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsExponentialGroup
 */
PdsExponential *pds_exponential_new(PdsRvReal Lambda)
{
	PdsExponential *X=NULL;
	
	if(Lambda<=0)	{return NULL;}

	X=(PdsExponential *)calloc(1,sizeof(PdsExponential));
	if(X==NULL)	{return NULL;}

	X->X1=pds_congruential_new();	
	if(X->X1==NULL)	{return NULL;}

	X->Lambda=Lambda;

	return X;
}


/** \fn int pds_exponential_init(PdsExponential *X, PdsRvReal x0)
 *  \brief Inicia la variable aleatorio exponencial,
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[in] x0 El valor inicial de la Variable aleatoria Exponencial.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL o x0<0.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_init(PdsExponential *X, PdsRvReal x0)
{
	PdsRvNaturalD x1;

	if(X==NULL)	{return FALSE;}
	if(x0<0)	{return FALSE;}

	x1=(PdsRvNaturalD)(PDS_RAND_MAX-PDS_RAND_MAX*exp(-X->Lambda*x0));

	pds_congruential_init(X->X1,x1);

	X->x=x0;

        return TRUE;
}


/** \fn int pds_exponential_get_value(PdsExponential *X, PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio exponencial,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *
 *  Para generar los elementos de la R.V. exponencial se usa una R.V. uniformemente 
 *  distribuida X1, de la siguiente manera: \f$ -\frac{ln(1-x1)}{\lambda} \f$ 
 *  \manonly ln(1-x1)/Lambda \endmanonly
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[out] x Un número con una pdf,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_get_value(PdsExponential *X, PdsRvReal *x)
{
	PdsRvNaturalD x1;

	if(X==NULL)	{*x=0;	return FALSE;}

	pds_congruential_get_value(X->X1,&x1);

	X->x=(LN_PDS_RAND_MAX-log((PdsRvRealD)(PDS_RAND_MAX-x1)))/X->Lambda;

	*x=X->x;

        return TRUE;
}


/** \fn int pds_exponential_get_last_value(PdsExponential *X, PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregado por la R.V. exponencial,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[out] x Un número con una pdf,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_get_last_value(PdsExponential *X, PdsRvReal *x)
{
	if(X==NULL)	{*x=0;	return FALSE;}

	*x=X->x;

        return TRUE;
}


/** \fn int pds_exponential_get_vector(PdsExponential *X, PdsVector *x)
 *  \brief Carga un vector con valores pseudo aleatorio exponenciales,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *
 *  Para generar los elementos de la R.V. exponencial se usa una R.V. uniformemente 
 *  distribuida X1, de la siguiente manera: \f$ -\frac{ln(1-x1)}{\lambda} \f$ 
 *  \manonly ln(1-x1)/Lambda \endmanonly
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[out] x Un vector con una pdf,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  Si X es NULL entonces el vector no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_get_vector(PdsExponential *X, PdsVector *x)
{
	PdsRvNaturalD x1;
	PdsRaNatural i;

	if(X==NULL)	{return FALSE;}
	if(x==NULL)	{return FALSE;}

	for(i=0;i<x->Nel;i++)
	{
		pds_congruential_get_value(X->X1,&x1);

		X->x=(LN_PDS_RAND_MAX-log((PdsRvRealD)(PDS_RAND_MAX-x1)))/X->Lambda;

		x->V[i]=X->x;
	}
        return TRUE;
}


/** \fn void pds_exponential_free(PdsExponential *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsExponential.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsExponentialGroup
 */
void pds_exponential_free(PdsExponential *X)
{
	if(X!=NULL)
	{
		pds_congruential_free(X->X1);

		free(X);
	}
}


/** \fn void pds_exponential_destroy(PdsExponential **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsExponential, y la carga con NULL.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsExponentialGroup
 */
void pds_exponential_destroy(PdsExponential **X)
{
	if((*X)!=NULL)
	{
		pds_congruential_free((*X)->X1);

		free(*X);
		(*X)=NULL;
	}
}

