/*
 * pdsgaussian.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#include <stdlib.h>
#include <math.h>

#include <pds/pdscongruential.h>
#include <pds/pdsuniform.h>
#include <pds/pdsgaussian.h>
#include <pds/pdsvector.h>


////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsGaussian                                            ////
////////////////////////////////////////////////////////////////////////////////


/** \fn PdsGaussian *pds_gaussian_new(PdsRvReal U,PdsRvReal Sigma)
 *  \brief Crea una variable aleatoria de tipo PdsGaussian.
 *  \param[in] U Es la media de la variable aleatoria gaussiana.
 *  \param[in] Sigma Es el desvio padron de la variable aleatoria gaussiana.
 *  \return Un puntero a la variable aleatoria de tipo Gauss,
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsGaussianGroup
 */
PdsGaussian *pds_gaussian_new(PdsRvReal U,PdsRvReal Sigma)
{
	PdsGaussian *X=NULL;
	
	X=(PdsGaussian *)calloc(1,sizeof(PdsGaussian));
	if(X==NULL)	{return NULL;}

	X->X1=pds_congruential_new();	
	if(X->X1==NULL)	{return NULL;}
	X->X2=pds_uniform_new(0,1);	
	if(X->X2==NULL)	{return NULL;}

	X->U=U;
	X->Sigma=Sigma;

	return X;
}



/** \fn int pds_gaussian_init(PdsGaussian *X, PdsRvReal x0)
 *  \brief Inicia la variable aleatoria Gaussiana,
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[in] x0 El valor inicial de la Variable aleatoria Gaussiana.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_init(PdsGaussian *X, PdsRvReal x0)
{
	PdsRvNaturalD x1;
	PdsRvReal x2;

	if(X==NULL)	{return FALSE;}

	x2=0.5;
	x1=(PdsRvNaturalD)(PDS_RAND_MAX-PDS_RAND_MAX*exp((-0.5)*((x0-X->U)/X->Sigma)*((x0-X->U)/X->Sigma)));

	pds_congruential_init(X->X1,x1);
	pds_uniform_init(X->X2,x2);

	X->x=x0;

        return TRUE;
}

/** \fn int pds_gaussian_get_value(PdsGaussian *X,PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio gaussiano, 
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly. 
 *  
 *  para generar los elementos de la R.V. gaussiana se usan dos R.V. uniformemente 
 *  distribuidas X1 y X2, de la siguiente manera:
 *  \f$ \sigma \sqrt{-2.0 ln(1-x1)}  cos(2.0  \pi  x2) + U \f$
 *  \manonly Sigma*sqrt(-2.0*ln(1-x1))*cos(2.0*pi*x2)+U \endmanonly
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[out] x x es cargado con un número con una pdf, \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_get_value(PdsGaussian *X,PdsRvReal *x)
{
	PdsRvNaturalD x1;
	PdsRvReal x2;

	if(X==NULL)	{*x=0;	return FALSE;}

	pds_congruential_get_value(X->X1,&x1);
	pds_uniform_get_value(X->X2,&x2);

	X->x=X->Sigma*sqrt(_2LN_PDS_RAND_MAX-2.0*log((PdsRvRealD)(PDS_RAND_MAX-x1)))*cos(2.0*M_PI*x2)+X->U;

	*x=X->x;	

        return TRUE;
}


/** \fn int pds_gaussian_get_last_value(PdsGaussian *X,PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregado por la R.V. gaussiana, 
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly. 
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[out] x x es cargado con un número con una pdf, \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_get_last_value(PdsGaussian *X,PdsRvReal *x)
{
	if(X==NULL)	{*x=0;	return FALSE;}

	*x=X->x;	

        return TRUE;
}


/** \fn int pds_gaussian_get_vector(PdsGaussian *X,PdsVector *x)
 *  \brief Carga un vector con valores pseudo aleatorio gaussianos, 
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly. 
 *  
 *  para generar los elementos de la R.V. gaussiana se usan dos R.V. uniformemente 
 *  distribuidas X1 y X2, de la siguiente manera:
 *  \f$ \sigma \sqrt{-2.0 ln(1-x1)}  cos(2.0  \pi  x2) + U \f$
 *  \manonly Sigma*sqrt(-2.0*ln(1-x1))*cos(2.0*pi*x2)+U \endmanonly
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[out] x Es el vector cargado con un número con una pdf, \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  Si X es NULL entonces x no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_get_vector(PdsGaussian *X,PdsVector *x)
{
	PdsRvNaturalD x1;
	PdsRvReal x2;
	PdsRaNatural i;

	if(X==NULL)	{return FALSE;}
	if(x==NULL)	{return FALSE;}

	for(i=0;i<x->Nel;i++)
	{
		pds_congruential_get_value(X->X1,&x1);
		pds_uniform_get_value(X->X2,&x2);

		X->x=X->Sigma*sqrt(_2LN_PDS_RAND_MAX-2.0*log((PdsRvRealD)(PDS_RAND_MAX-x1)))*cos(2.0*M_PI*x2)+X->U;

		x->V[i]=X->x;	
	}
        return TRUE;
}


/** \fn void pds_gaussian_free(PdsGaussian *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsGaussian.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsGaussianGroup
 */
void pds_gaussian_free(PdsGaussian *X)
{
	if(X!=NULL)
	{
		pds_congruential_free(X->X1);
		pds_uniform_free(X->X2);

		free(X);
	}
}


/** \fn void pds_gaussian_destroy(PdsGaussian **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsGaussian, y la carga con NULL.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsGaussianGroup
 */
void pds_gaussian_destroy(PdsGaussian **X)
{
	if((*X)!=NULL)
	{
		pds_congruential_free((*X)->X1);
		pds_uniform_free((*X)->X2);

		free(*X);
		(*X)=NULL;
	}
}

