/*
 * pdspoisson.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#include <stdlib.h>
#include <math.h>

#include <pds/pdsuniform.h>
#include <pds/pdspoisson.h>
#include <pds/pdsvector.h>


////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsPoisson                                             ////
////////////////////////////////////////////////////////////////////////////////


/** \fn PdsPoisson *pds_poisson_new(PdsRvReal Lambda)
 *  \brief Crea una variable aleatoria de tipo PdsPoisson.
 *  \param[in] Lambda La media de la variable aleatoria poissoniana (Lambda<100).
 *  \return Un puntero a la variable aleatoria de tipo Poisson,
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
 *  La función retorna NULL en caso de error. 
 *  \ingroup PdsPoissonGroup
 */
PdsPoisson *pds_poisson_new(PdsRvReal Lambda)
{
	PdsPoisson *X=NULL;
	
	X=(PdsPoisson *)calloc(1,sizeof(PdsPoisson));
	if(X==NULL)	{return NULL;}

	X->X1=pds_uniform_new(0,1);	
	if(X->X1==NULL)	{return NULL;}

	X->Lambda=Lambda;

	return X;
}

/** \fn int pds_poisson_init(PdsPoisson *X, PdsRvInteger k0)
 *  \brief Inicia la variable aleatoria Poisson,
 *  \param[in] X Variable aleatoria Poisson.
 *  \param[in] k0 El valor que seteará un valor inicial a la Variable aleatoria Poisson.
 *             Este no será el valor inicial.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL o k0<0.
 *  \ingroup PdsPoissonGroup
 */
int pds_poisson_init(PdsPoisson *X, PdsRvInteger k0)
{

	if(X==NULL)	{return FALSE;}
	if(k0<0)	{return FALSE;}

	pds_congruential_init(X->X1->X1,(PdsRvNaturalD)(k0%PDS_RAND_MAX));

	X->k=k0;

        return TRUE;
}

/** \fn int pds_poisson_get_value(PdsPoisson *X, PdsRvInteger *x)
 *  \brief Retorna un número pseudo aleatorio poissoniano,
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
 *
 *  Esta R.V. está generada mediante el uso del algoritmo de Knuth.
 *  \param[in] X Variable aleatoria Poissoniana.
 *  \param[out] x Un número con una pdf, 
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly .
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo Fue bien o FASE sino. Ejemplo X==NULL.
 *  \ingroup PdsPoissonGroup
 */
int pds_poisson_get_value(PdsPoisson *X, PdsRvInteger *x)
{
	PdsRvReal x1,p=1.0;
	PdsRvInteger K=0;
	if(X==NULL)	{*x=0;	return FALSE;}
	do{
		K=K+1;
		pds_uniform_get_value(X->X1,&x1);
		p=p*x1;
	}while(p > exp(-X->Lambda));

	X->k=K-1;
	*x=X->k;

        return TRUE;
}


/** \fn int pds_poisson_get_last_value(PdsPoisson *X,PdsRvInteger *x)
 *  \brief Retorna el último número pseudo aleatorio de la R.V. poissoniana,
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
 *  \param[in] X Variable aleatoria Poissoniana.
 *  \param[out] x Un número con una pdf, 
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly .
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo Fue bien o FASE sino. Ejemplo X==NULL.
 *  \ingroup PdsPoissonGroup
 */
int pds_poisson_get_last_value(PdsPoisson *X,PdsRvInteger *x)
{
	if(X==NULL)	{*x=0;	return FALSE;}

	*x=X->k;

        return TRUE;
}


/** \fn void pds_poisson_free(PdsPoisson *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsPoisson.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsPoissonGroup
 */
void pds_poisson_free(PdsPoisson *X)
{
	if(X!=NULL)
	{
		pds_uniform_free(X->X1);

		free(X);
	}
}


/** \fn void pds_poisson_destroy(PdsPoisson **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsPoisson, y la carga con NULL.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsPoissonGroup
 */
void pds_poisson_destroy(PdsPoisson **X)
{
	if((*X)!=NULL)
	{
		pds_uniform_free((*X)->X1);

		free(*X);
		(*X)=NULL;
	}
}

