/*
 * pdsrayleigh.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#include <stdlib.h>
#include <math.h>

#include <pds/pdscongruential.h>
#include <pds/pdsrayleigh.h>
#include <pds/pdsvector.h>


////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsRayleigh                                            ////
////////////////////////////////////////////////////////////////////////////////


/** \fn PdsRayleigh *pds_rayleigh_new(PdsRvReal Sigma)
 *  \brief Crea una variable aleatoria de tipo PdsRayleigh.
 *  \param[in] Sigma Es la moda de la variable aleatoria rayleighiana.
 *  \return Un puntero a la variable aleatoria de tipo Rayleigh, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsRayleighGroup
 */
PdsRayleigh *pds_rayleigh_new(PdsRvReal Sigma)
{
	PdsRayleigh *X=NULL;
	
	X=(PdsRayleigh *)calloc(1,sizeof(PdsRayleigh));
	if(X==NULL)	{return NULL;}

	X->X1=pds_congruential_new();	
	if(X->X1==NULL)	{return NULL;}

	X->Sigma=Sigma;

	return X;
}



/** \fn int pds_rayleigh_init(PdsRayleigh *X, PdsRvReal x0)
 *  \brief Inicia la variable aleatorio Rayleigh,
 *  \param[in] X Variable aleatoria Rayleigh.
 *  \param[in] x0 El valor inicial de la Variable aleatoria Rayleigh.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL o x0<0.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_init(PdsRayleigh *X, PdsRvReal x0)
{
	PdsRvNaturalD x1;

	if(X==NULL)	{return FALSE;}
	if(x0<0)	{return FALSE;}

	x1=(PdsRvNaturalD)(PDS_RAND_MAX-PDS_RAND_MAX*exp((-0.5)*(x0/X->Sigma)*(x0/X->Sigma)));

	pds_congruential_init(X->X1,x1);

	X->x=x0;

        return TRUE;
}


/** \fn int pds_rayleigh_get_value(PdsRayleigh *X,PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio rayleighiano, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *
 *  Para generar la variable aleatoria Rayleighiana se usa una variable aleatoria
 *  uniformemente distribuida X1 entre [0,1), de la siguiente manera:
 *  \f$ X=\sigma \sqrt{-2.0 ln(1-x1)} \f$ \manonly sigma*sqrt(-2.0*ln(1-x1)) \endmanonly.
 *  \param[in] X Variable aleatoria Rayleighiana.
 *  \param[out] x Es un número con una pdf,
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_get_value(PdsRayleigh *X,PdsRvReal *x)
{
	PdsRvNaturalD x1;
	if(X==NULL)	{*x=0;	return FALSE;}

	pds_congruential_get_value(X->X1,&x1);

	X->x=X->Sigma*sqrt(_2LN_PDS_RAND_MAX-2.0*log((PdsRvRealD)(PDS_RAND_MAX-x1)));

	*x=X->x;

        return TRUE;
}


/** \fn int pds_rayleigh_get_last_value(PdsRayleigh *X, PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregada por la R.V. rayleighiana, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  \param[in] X Variable aleatoria Rayleighiana.
 *  \param[out] x Es un número con una pdf,
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_get_last_value(PdsRayleigh *X, PdsRvReal *x)
{
	if(X==NULL)	{*x=0;	return FALSE;}

	*x=X->x;

        return TRUE;
}


/** \fn int pds_rayleigh_get_vector(PdsRayleigh *X,PdsVector *x)
 *  \brief Carga un vector con números pseudo aleatorios rayleighianos, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *
 *  Para generar la variable aleatoria Rayleighiana se usa una variable aleatoria
 *  uniformemente distribuida X1 entre [0,1), de la siguiente manera:
 *  \f$ X=\sigma \sqrt{-2.0 ln(1-x1)} \f$ \manonly sigma*sqrt(-2.0*ln(1-x1)) \endmanonly.
 *  \param[in] X Variable aleatoria Rayleighiana.
 *  \param[out] x Es un vector con una pdf,
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  Si X es NULL entonces El vector no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_get_vector(PdsRayleigh *X,PdsVector *x)
{
	PdsRvNaturalD x1;
	PdsRaNatural i;

	if(X==NULL)	{return FALSE;}
	if(x==NULL)	{return FALSE;}

	for(i=0;i<x->Nel;i++)
	{
		pds_congruential_get_value(X->X1,&x1);

		X->x=X->Sigma*sqrt(_2LN_PDS_RAND_MAX-2.0*log((PdsRvRealD)(PDS_RAND_MAX-x1)));

		x->V[i]=X->x;
	}
        return TRUE;
}


/** \fn void pds_rayleigh_free(PdsRayleigh *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsRayleigh.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsRayleighGroup
 */
void pds_rayleigh_free(PdsRayleigh *X)
{
	if(X!=NULL)
	{
		pds_congruential_free(X->X1);

		free(X);
	}
}


/** \fn void pds_rayleigh_destroy(PdsRayleigh **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsRayleigh, y la carga con NULL.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsRayleighGroup
 */
void pds_rayleigh_destroy(PdsRayleigh **X)
{
	if((*X)!=NULL)
	{
		pds_congruential_free((*X)->X1);

		free(*X);
		(*X)=NULL;
	}
}

