/*
 * pdsuniform.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

#include <stdlib.h>
#include <math.h>

#include <pds/pdscongruential.h>
#include <pds/pdsuniform.h>
#include <pds/pdsvector.h>


////////////////////////////////////////////////////////////////////////////////
////  Trabajando con PdsUniform                                             ////
////////////////////////////////////////////////////////////////////////////////


/** \fn PdsUniform *pds_uniform_new(PdsRvReal A,PdsRvReal B)
 *  \brief Crea una variable aleatoria de tipo PdsUniform.
 *  \param[in] A Rango inicial izquierdo de la variable aleatoria.
 *  \param[in] B Rango final derecho de la variable aleatoria.
 *  \return Un puntero a la variable aleatoria de tipo PdsUniform.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsUniformGroup
 */
PdsUniform *pds_uniform_new(PdsRvReal A,PdsRvReal B)
{
	PdsUniform *X=NULL;
	
	X=(PdsUniform *)calloc(1,sizeof(PdsUniform));
	if(X==NULL)	{return NULL;}

	X->X1=pds_congruential_new();	
	if(X->X1==NULL)	{return NULL;}

	X->A=A;
	X->B=B;

	return X;
}


/** \fn int pds_uniform_init(PdsUniform *X, PdsRvReal x0)
 *  \brief Inicializa la variable pseudo aleatorio y uniforme entre [A,B).
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[in] x0 X{inicial}=x0, si esta fuera de [A,B),X{inicial}=A.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_init(PdsUniform *X, PdsRvReal x0)
{
	PdsRvNaturalD x1;

	if(X==NULL)	return FALSE;

	if((x0 < X->A)||(x0 >= X->B))	x0=X->A;

	x1=((PdsRvReal)(x0 - X->A)/(PdsRvReal)(X->B - X->A))*(PDS_RAND_MAX-64);

	pds_congruential_init(X->X1,x1);

	X->x=x0;

	return TRUE;
}


/** \fn int pds_uniform_get_value(PdsUniform *X, PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio y uniforme entre [A,B), 
 *        con PDS_RAND_MAX divisiones de resolución.
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[out] x x es cargado con un número entre [A,B). Incluye A pero no  B.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_get_value(PdsUniform *X, PdsRvReal *x)
{
	PdsRvNaturalD x1;

	if(X==NULL)	{*x=0;	return FALSE;}

	pds_congruential_get_value(X->X1,&x1);

	X->x=(((PdsRvReal)x1)/((PdsRvReal)(PDS_RAND_MAX+64)))*(X->B-X->A)+X->A;

	*x=X->x;

	return TRUE;
}


/** \fn int pds_uniform_get_last_value(PdsUniform *X, PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregado por la R.V. uniforme entre [A,B), 
 *        con PDS_RAND_MAX divisiones de resolución.
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[out] x x es cargado con un número entre [A,B). Incluye A pero no  B.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_get_last_value(PdsUniform *X, PdsRvReal *x)
{
	if(X==NULL)	{*x=0;	return FALSE;}

	*x=X->x;

	return TRUE;
}


/** \fn int pds_uniform_get_vector(PdsUniform *X, PdsVector *x)
 *  \brief Retorna un vector con valores pseudo aleatorio y uniforme entre [A,B), 
 *        con PDS_RAND_MAX divisiones de resolución.
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[out] x x es cargado con números entre [A,B). Incluye A pero no  B.
 *  Si X es NULL entonces el vector no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_get_vector(PdsUniform *X, PdsVector *x)
{
	PdsRvNaturalD x1;
	PdsRaNatural i;

	if(X==NULL)	{return FALSE;}
	if(x==NULL)	{return FALSE;}

	for(i=0;i<x->Nel;i++)
	{

		pds_congruential_get_value(X->X1,&x1);
	
		X->x=(((PdsRvReal)x1)/((PdsRvReal)(PDS_RAND_MAX+64)))*(X->B-X->A)+X->A;

		x->V[i]=X->x;
	}
	return TRUE;
}

/** \fn void pds_uniform_free(PdsUniform *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsUniform.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsUniformGroup
 */
void pds_uniform_free(PdsUniform *X)
{
	if(X!=NULL)
	{
		pds_congruential_free(X->X1);
		free(X);
	}
}


/** \fn void pds_uniform_destroy(PdsUniform **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsUniform, y la carga con NULL.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsUniformGroup
 */
void pds_uniform_destroy(PdsUniform **X)
{
	if((*X)!=NULL)
	{
		pds_congruential_free((*X)->X1);

		free(*X);
		(*X)=NULL;
	}
}


