/*
 * extras.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdscoin.h
 *  \author Fernando Pujaico Rivera
 *  \date 03-11-2012
 *  \brief Funciones que trabajan con variables aleatorias de Bernoulli (pseudoaleatorias).
 *  
 *  <br><br>Estas funciones trabajan con una variable aleatoria (R.V) X de tipo Bernoulli.
 *  Esta R.V. está generada mediante el método congruencial mixto (2^N).
 */

#ifndef __PDSCOIN_H__
#define __PDSCOIN_H__

#include <pds/pdsrv.h>
#include <pds/pdsra.h>
//#include <pds/pdsba.h>

#ifdef __cplusplus
extern "C" {
#endif 

/** \defgroup PdsCoinGroup Funciones del módulo PdsCoin.
 * @{
 */

/*! Minima probabilidad detectable y generable, es decir una probabilidad p,
 *  p< PDS_MIN_UNIFORM_PROB será considerada como cero; dada una probabilidad
 *  cualquiera p0, la probabilidad p+p0 será considerada como p0.
 *  \ingroup PdsCoinGroup
 */
#define PDS_MIN_COIN_PROB 1.0/PDS_RAND_MAX 


/*! \struct PdsCoin
 *  \brief La estructura tipo  PdsCoin.
 *  Esta estructura genera una variable aleatoria de Bernoulli.
 *  Para usar, incluir pdsrv/pdsrv.h.
 *  La función de densidad de probabilidad es:
 *  \f[ f_x(x=1)=p, f_x(x=0)=1-p \f] 
 *  \manonly fx(x=1)=p, fx(x=0)=1-p, fx(x=other)=0  \endmanonly
 *  \ingroup PdsCoinGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Variable aleatoria de tipo Congruential.
         */
	PdsCongruential *X1;
	/*! Ultimo valor obtenido en la variable aleatoria.
         */
	PdsRvReal x;
	/*! Probabilidad de obtener un 1 con la variable aleatoria.
         */
	PdsRvReal p;
}PdsCoin;

/** \fn PdsCoin *pds_coin_new(PdsRvReal p)
 *  \brief Crea una variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad p de obtener un 1 al ser jugada.
 *  \param[in] p Probabilidad de la moneda de obtener un 1.
 *  \return Un puntero a la variable aleatoria de tipo PdsCoin.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsCoinGroup
 */
PdsCoin *pds_coin_new(PdsRvReal p);

/** \fn int pds_coin_init(PdsCoin *X, PdsRvReal x0)
 *  \brief Inicia con un valor x0 {-1,+1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[in] x0 Valor inicial de la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_init(PdsCoin *X, PdsRvReal x0);

/** \fn int pds_coin_set_p(PdsCoin *X, PdsRvReal p)
 *  \brief Inicia con una probabilidad de p a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[in] p Valor de probabilidad de la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_set_p(PdsCoin *X, PdsRvReal p);

/** \fn int pds_coin_get_value(PdsCoin *X, PdsRvReal *x)
 *  \brief Pide un valor {-1,1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Valor de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_value(PdsCoin *X, PdsRvReal *x);

/** \fn int pds_coin_get_bit(PdsCoin *X, PdsRvByte *x)
 *  \brief Pide un valor {0,1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Valor de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_bit(PdsCoin *X, PdsRvByte *x);

/** \fn int pds_coin_get_last_value(PdsCoin *X, PdsRvReal *x)
 *  \brief Pide el último valor {-1,1} obtenido de la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Ultimo valor obtenido de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_last_value(PdsCoin *X, PdsRvReal *x);

/** \fn int pds_coin_get_last_bit(PdsCoin *X, PdsRvByte *x)
 *  \brief Pide el último valor {0,1} obtenido de la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] x Ultimo valor obtenido de jugar la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_last_bit(PdsCoin *X, PdsRvByte *x);

/** \fn int pds_coin_get_vector(PdsCoin *X, PdsVector *V)
 *  \brief Pide un vector de valores {-1,+1} a la variable aleatoria de tipo PdsCoin.
 *
 *  Una variable de tipo PdsCoin representa una moneda de valores {0,1}
 *  con una probabilidad X->p de obtener un valor de 1 al ser jugada.
 *  \param[in] X Variable aleatoria de tipo PdsCoin.
 *  \param[out] V Vector a cargar con valores de la variable aleatoria de tipo PdsCoin.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL o V==NULL.
 *  \ingroup PdsCoinGroup
 */
int pds_coin_get_vector(PdsCoin *X, PdsVector *V);

/** \fn void pds_coin_free(PdsCoin *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsCoin.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCoinGroup
 */
void pds_coin_free(PdsCoin *X);

/** \fn void pds_coin_destroy(PdsCoin **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsCoin, y la carga con NULL.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCoinGroup
 */
void pds_coin_destroy(PdsCoin **X);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif /* __PDSCOIN_H__ */
