/*
 * pdscongruential.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdscongruential.h
 *  \author Fernando Pujaico Rivera
 *  \date 01-04-2011
 *  \brief Funciones que trabajan con secuencias aleatorias congruenciales (pseudoaleatorias).
 *  
 *  <br><br>Estas funciones trabajan con una secuencia aleatoria X de tipo congruencial
 *  distribuida entre [0,PDS_RAND_MAX), esta secuencia está generada mediante el método
 *  congruencial mixto (2^N).
 */

#ifndef __PDSCONGRUENTIAL_H__
#define __PDSCONGRUENTIAL_H__

#include <stdio.h>
#include <pds/pdsrvglobal.h>
#include <pds/pdsvector.h>

#ifdef __cplusplus
extern "C" {
#endif 

/** \defgroup PdsCongruentialGroup Funciones del módulo PdsCongruential.
 * @{
 */

/*! Máximo número de elementos de las secuencias aleatorias. 
 *  PDS_RAND_MAX es un número primo o una potencia de un número primo.
 *  Este número indica que la secuencia generada tiene un periodo de
 *  PDS_RAND_MAX>. Se ha usado el método congruencial multiplicativo 2^N,
 *  N=31, para generar la variable aleatoria congruentialemente distribuida.
 *  \ingroup PdsCongruentialGroup
 */
#define PDS_RAND_MAX 2147483648UL

/*! Logaritmo natural del máximo número de elementos de las secuencias aleatorias. 
 *  ln(PDS_RAND_MAX).
 *  \ingroup PdsCongruentialGroup
 */
#define LN_PDS_RAND_MAX  21.487562597

/*! Dos veces el logaritmo natural del máximo número de elementos de las 
 *  secuencias aleatorias. 2 ln(PDS_RAND_MAX).
 *  \ingroup PdsCongruentialGroup
 */
#define _2LN_PDS_RAND_MAX 42.975125195


/*! \struct PdsCongruential
 *  \brief La estructura tipo  PdsCongruential.
 *  Esta estructura genera una secuencia aleatoria congruential.
 *  Esta secuencia está generada mediante el método congruencial mixto para 2^N, N=31.
 *  Para usar, incluir pdsrv/pdsrv.h.
 *  \ingroup PdsCongruentialGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! La semilla inicial de la secuencia aleatoria. 0<x0<PDS_RAND_MAX
         */
	PdsRvNaturalD x0;

	/*! El multiplicador de la secuencia aleatoria. 1<a<PDS_RAND_MAX,
	 * a-1 debe ser múltiplo de los factores primos de PDS_RAND_MAX,
	 * es decir múltiplo de 2. Si PDS_RAND_MAX es multiplo de 4, a-1 
	 * también debe serlo.
	 */
	PdsRvNaturalD a;

	/*! El incremento de la secuencia aleatoria. 1 < c < PDS_RAND_MAX,
	 *  ademas c y PDS_RAND_MAX son primos relativos.
	 */
	PdsRvNaturalD c;

	/*! La salida entera de la secuencia aleatoria. 
         * 0 <= xn < PDS_RAND_MAX. 
         */
	PdsRvNaturalD xn;
}PdsCongruential;

/** \fn PdsCongruential *pds_congruential_new(void)
 *  \brief Crea una secuencia aleatoria de tipo PdsCongruential.
 *  \return Un puntero a la estructura de la secuencia aleatoria de tipo PdsCongruential.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsCongruentialGroup
 */
PdsCongruential *pds_congruential_new(void);

/** \fn int pds_congruential_init(PdsCongruential *X,PdsRvNaturalD x0)
 *  \brief Inicializa la secuencia aleatoria de tipo PdsCongruential con x0.
 *  \param[in] X Variable secuencia aleatoria congruential.
 *  \param[in] x0 X{inicial}=x0 \% PDS_RAND_MAX.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsCongruentialGroup
 */
int pds_congruential_init(PdsCongruential *X,PdsRvNaturalD x0);

/** \fn int pds_congruential_get_value(PdsCongruential *X,PdsRvNaturalD *x)
 *  \brief Retorna un número de la secuencia pseudo aleatorio entre [0,PDS_RAND_MAX).
 *  \param[in] X Variable secuencia aleatoria congruential.
 *  \param[out] x x es cargado con un número entre [0,PDS_RAND_MAX). Incluye 0 pero no  PDS_RAND_MAX.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsCongruentialGroup
 */
int pds_congruential_get_value(PdsCongruential *X,PdsRvNaturalD *x);


/** \fn int pds_congruential_get_last_value(PdsCongruential *X,PdsRvNaturalD *x)
 *  \brief Retorna el último número entregado por la secuencia pseudo aleatorio entre [0,PDS_RAND_MAX).
 *  \param[in] X Variable secuencia aleatoria congruential.
 *  \param[out] x x es cargado con un número entre [0,PDS_RAND_MAX). Incluye 0 pero no  PDS_RAND_MAX.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsCongruentialGroup
 */
int pds_congruential_get_last_value(PdsCongruential *X,PdsRvNaturalD *x);


/** \fn void pds_congruential_free(PdsCongruential *X)
 *  \brief Libera la variable secuencia aleatoria de tipo puntero PdsCongruential.
 *  \param[in] X La variable secuencia aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCongruentialGroup
 */
void pds_congruential_free(PdsCongruential *X);



/** \fn void pds_congruential_destroy(PdsCongruential **X)
 *  \brief Libera la variable secuencia aleatoria de tipo puntero PdsCongruential, y la carga con NULL.
 *  \param[in] X La variable secuencia aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsCongruentialGroup
 */
void pds_congruential_destroy(PdsCongruential **X);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif /* __PDSCONGRUENTIAL_H__ */ 

