/*
 * pdsexponential.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdsexponential.h
 *  \author Fernando Pujaico Rivera
 *  \date 01-04-2011
 *  \brief Funciones que trabajan con variables aleatorias exponenciales (pseudoaleatorias).
 *  
 *  <br><br>Estas funciones trabajan con una variable aleatoria (R.V) X de tipo exponencial
 *  de media 1/Lambda (E[X]=1/Lambda) e varianza 1/Lambda^2 (E[X^2-1/Lambda^2]=1/Lambda^2), 
 *  esta VA esta generada mediante el uso de una variable aleatoria uniformemente 
 *  distribuida creada con el método congruencial mixto 2^N. 
 */

#ifndef __PDSEXPONENTIAL_H__
#define __PDSEXPONENTIAL_H__

#include <stdio.h>
#include <pds/pdsrvglobal.h>
#include <pds/pdscongruential.h>
#include <pds/pdsvector.h>

#ifdef __cplusplus
extern "C" {
#endif 

/** \defgroup PdsExponentialGroup Funciones del módulo PdsExponential.
 * @{
 */


/*! \struct PdsExponential
 *  \brief La estructura tipo  PdsExponential.
 *  Esta estructura genera una variable aleatoria exponencial.
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly
 *  Para usar incluir pdsrv/pdsrv.h.
 *  La función de densidad de probabilidad es:
 *  \f[ f_x(x)=\lambda e^{-\lambda x} , x \in [0,\infty) \f]
 *  \manonly fx(x)=lambda e^{-lambda x} , x [0,oo)  \endmanonly
 *  \ingroup PdsExponentialGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Una secuencia aleatoria congruencial*/
	PdsCongruential *X1;

	/*! 1/Lambda es la media de la variable aleatoria exponencial. */
	PdsRvReal Lambda;

	/*! La salida de la variable aleatoria exponencial, Exponential(Lambda)
	 * \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly
	 */
	PdsRvReal x;
}PdsExponential;

/** \fn PdsExponential *pds_exponential_new(PdsRvReal Lambda)
 *  \brief Crea una variable aleatoria de tipo PdsExponential.
 *  \param[in] Lambda La inversa de la media de la variable aleatoria exponencial.
 *		si Lambda es menor igual a cero se retorna NULL.
 *  \return Un puntero a la variable aleatoria de tipo Exponential,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsExponentialGroup
 */
PdsExponential *pds_exponential_new(PdsRvReal Lambda);


/** \fn int pds_exponential_init(PdsExponential *X, PdsRvReal x0)
 *  \brief Inicia la variable aleatorio exponencial,
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[in] x0 El valor inicial de la Variable aleatoria Exponencial.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL o x0<0.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_init(PdsExponential *X, PdsRvReal x0);


/** \fn int pds_exponential_get_value(PdsExponential *X, PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio exponencial,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *
 *  Para generar los elementos de la R.V. exponencial se usa una R.V. uniformemente 
 *  distribuida X1, de la siguiente manera: \f$ -\frac{ln(1-x1)}{\lambda} \f$ 
 *  \manonly ln(1-x1)/Lambda \endmanonly
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[out] x Un número con una pdf,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_get_value(PdsExponential *X, PdsRvReal *x);


/** \fn int pds_exponential_get_last_value(PdsExponential *X, PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregado por la R.V. exponencial,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[out] x Un número con una pdf,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_get_last_value(PdsExponential *X, PdsRvReal *x);


/** \fn int pds_exponential_get_vector(PdsExponential *X, PdsVector *x)
 *  \brief Carga un vector con valores pseudo aleatorio exponenciales,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *
 *  Para generar los elementos de la R.V. exponencial se usa una R.V. uniformemente 
 *  distribuida X1, de la siguiente manera: \f$ -\frac{ln(1-x1)}{\lambda} \f$ 
 *  \manonly ln(1-x1)/Lambda \endmanonly
 *  \param[in] X Variable aleatoria Exponencial.
 *  \param[out] x Un vector con una pdf,
 *  \f$ Exponential(\lambda) \f$ \manonly Exponential(Lambda) \endmanonly.
 *  Si X es NULL entonces el vector no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsExponentialGroup
 */
int pds_exponential_get_vector(PdsExponential *X, PdsVector *x);


/** \fn void pds_exponential_free(PdsExponential *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsExponential.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsExponentialGroup
 */
void pds_exponential_free(PdsExponential *X);


/** \fn void pds_exponential_destroy(PdsExponential **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsExponential, y la carga con NULL.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsExponentialGroup
 */
void pds_exponential_destroy(PdsExponential **X);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif /* __PDSEXPONENTIAL_H__ */ 

