/*
 * pdsgaussian.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdsgaussian.h
 *  \author Fernando Pujaico Rivera
 *  \date 01-04-2011
 *  \brief Funciones que trabajan con variables aleatorias gaussianas (pseudoaleatorias).
 *  
 *  <br><br>Estas funciones trabajan con una variable aleatoria (R.V.) X de tipo gaussiana
 *  de media U (E[X]=U) e varianza Sigma^2 (E[(X-U)^2]=Sigma^2), esta R.V. está generada mediante 
 *  el uso de dos variables aleatorias uniformemente distribuidas creadas con el método
 *  congruencial mixto 2^N.
 */

#ifndef __PDSGAUSSIAN_H__
#define __PDSGAUSSIAN_H__

#include <stdio.h>
#include <pds/pdsrvglobal.h>
#include <pds/pdscongruential.h>
#include <pds/pdsuniform.h>
#include <pds/pdsvector.h>

#ifdef __cplusplus
extern "C" {
#endif 

/** \defgroup PdsGaussianGroup Funciones del módulo PdsGaussian.
 * @{
 */



/*! \struct PdsGaussian
 *  \brief La estructura tipo  PdsGaussian.
 *  Esta estructura genera una variable aleatoria gaussiana normalizada,
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  Para usar incluir pdsrv/pdsrv.h.
 *  La función de densidad de probabilidad es:
 *  \f[ f_x(x)={\frac{1}{\sqrt{2 \pi {\sigma}^2}}e^{\frac{-(x-U)^2}{2\sigma^2}}} \f]
 *  \manonly fx(x)=N(U,Sigma^2)  \endmanonly
 *  \ingroup PdsGaussianGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Una secuencia aleatoria congruencial*/
	PdsCongruential *X1;

	/*! Una variable aleatoria uniformemente distribuida entre [0,1). */
	PdsUniform *X2;

	/*! La media de la variable aletoria gaussiana. */
	PdsRvReal U;

	/*! El desvío padrón de la variable aletoria gaussiana. */
	PdsRvReal Sigma;

	/*! La salida de la variable aletoria gaussiana, 
         *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly. 
         */
	PdsRvReal x;
}PdsGaussian;

/** \fn PdsGaussian *pds_gaussian_new(PdsRvReal U,PdsRvReal Sigma)
 *  \brief Crea una variable aleatoria de tipo PdsGaussian.
 *  \param[in] U Es la media de la variable aleatoria gaussiana.
 *  \param[in] Sigma Es el desvio padron de la variable aleatoria gaussiana.
 *  \return Un puntero a la variable aleatoria de tipo Gauss,
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsGaussianGroup
 */
PdsGaussian *pds_gaussian_new(PdsRvReal U,PdsRvReal Sigma);

/** \fn int pds_gaussian_init(PdsGaussian *X, PdsRvReal x0)
 *  \brief Inicia la variable aleatoria Gaussiana,
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[in] x0 El valor inicial de la Variable aleatoria Gaussiana.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_init(PdsGaussian *X, PdsRvReal x0);


/** \fn int pds_gaussian_get_value(PdsGaussian *X,PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio gaussiano, 
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly. 
 *  
 *  para generar los elementos de la R.V. gaussiana se usan dos R.V. uniformemente 
 *  distribuidas X1 y X2, de la siguiente manera:
 *  \f$ \sigma \sqrt{-2.0 ln(1-x1)}  cos(2.0  \pi  x2) + U \f$
 *  \manonly Sigma*sqrt(-2.0*ln(1-x1))*cos(2.0*pi*x2)+U \endmanonly
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[out] x x es cargado con un número con una pdf, \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_get_value(PdsGaussian *X,PdsRvReal *x);


/** \fn int pds_gaussian_get_last_value(PdsGaussian *X,PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregado por la R.V. gaussiana, 
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly. 
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[out] x x es cargado con un número con una pdf, \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_get_last_value(PdsGaussian *X,PdsRvReal *x);


/** \fn int pds_gaussian_get_vector(PdsGaussian *X,PdsVector *x)
 *  \brief Carga un vector con valores pseudo aleatorio gaussianos, 
 *  \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly. 
 *  
 *  para generar los elementos de la R.V. gaussiana se usan dos R.V. uniformemente 
 *  distribuidas X1 y X2, de la siguiente manera:
 *  \f$ \sigma \sqrt{-2.0 ln(1-x1)}  cos(2.0  \pi  x2) + U \f$
 *  \manonly Sigma*sqrt(-2.0*ln(1-x1))*cos(2.0*pi*x2)+U \endmanonly
 *  \param[in] X Variable aleatoria Gaussiana.
 *  \param[out] x Es el vector cargado con un número con una pdf, \f$\mathit{N}(U,\sigma^2)\f$ \manonly N(U,Sigma^2) \endmanonly.
 *  Si X es NULL entonces x no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X==NULL.
 *  \ingroup PdsGaussianGroup
 */
int pds_gaussian_get_vector(PdsGaussian *X,PdsVector *x);


/** \fn void pds_gaussian_free(PdsGaussian *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsGaussian.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsGaussianGroup
 */
void pds_gaussian_free(PdsGaussian *X);


/** \fn void pds_gaussian_destroy(PdsGaussian **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsGaussian, y la carga con NULL.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsGaussianGroup
 */
void pds_gaussian_destroy(PdsGaussian **X);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif /* __PDSGAUSSIAN_H__ */ 

