/*
 * pdspoisson.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdspoisson.h
 *  \author Fernando Pujaico Rivera
 *  \date 01-04-2011
 *  \brief Funciones que trabajan con variables aleatorias poissonianas (pseudoaleatorias).
 *
 *  <br><br>Una simple manera de generar números Poisson-distributed aleatorios es dado por Knuth.
 *  Las siguientes funciones trabajan con una variable aleatoria (R.V,) X de tipo poissoniana
 *  de media Lambda (E[X]=Lambda) e varianza Lambda (E[X^2-Lambda^2]=Lambda), esta R.V. 
 *  está generada mediante el uso del algoritmo de Knuth.
 */

#ifndef __PDSPOISSON_H__
#define __PDSPOISSON_H__

#include <stdio.h>
#include <pds/pdsrvglobal.h>
#include <pds/pdsuniform.h>
#include <pds/pdsvector.h>

#ifdef __cplusplus
extern "C" {
#endif 

/** \defgroup PdsPoissonGroup Funciones del módulo PdsPoisson.
 * @{
 */



/*! \struct PdsPoisson
 *  \brief La estructura tipo  PdsPoisson.
 *  Esta estructura genera una variable aleatoria poissoniana, 
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
 *  Para usar incluir pdsrv/pdsrv.h.
 *  La función de densidad de probabilidad es:
 *  \f[ f_k(k;\lambda)= \frac{ e^{-\lambda} \lambda^k}{k!}, k \in [0,\infty) \f]
 *  \manonly f_k(k;lambda)=  e^{-lambda} lambda^k / k!, k en [0,oo)  \endmanonly
 *  \ingroup PdsPoissonGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Una variable aleatoria uniformemente distribuida entre [0,1)*/
	PdsUniform *X1;

	/*! Lambda es la media de la variable aleatoria poissoniana. */
	PdsRvReal Lambda;

	/*! La salida de la variable aleatoria poissoniana,
	 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
	 */
	PdsRvInteger k;
}PdsPoisson;

/** \fn PdsPoisson *pds_poisson_new(PdsRvReal Lambda)
 *  \brief Crea una variable aleatoria de tipo PdsPoisson.
 *  \param[in] Lambda La media de la variable aleatoria poissoniana.
 *  \return Un puntero a la variable aleatoria de tipo Poisson,
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
 *  La función retorna NULL en caso de error. 
 *  \ingroup PdsPoissonGroup
 */
PdsPoisson *pds_poisson_new(PdsRvReal Lambda);


/** \fn int pds_poisson_init(PdsPoisson *X, PdsRvInteger k0)
 *  \brief Inicia la variable aleatoria Poisson,
 *  \param[in] X Variable aleatoria Poisson.
 *  \param[in] k0 El valor que seteará un valor inicial a la Variable aleatoria Poisson.
 *             Este no será el valor inicial.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL o k0<0.
 *  \ingroup PdsPoissonGroup
 */
int pds_poisson_init(PdsPoisson *X, PdsRvInteger k0);


/** \fn int pds_poisson_get_value(PdsPoisson *X, PdsRvInteger *x)
 *  \brief Retorna un número pseudo aleatorio poissoniano,
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
 *
 *  Esta R.V. está generada mediante el uso del algoritmo de Knuth.
 *  \param[in] X Variable aleatoria Poissoniana.
 *  \param[out] x Un número con una pdf, 
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly .
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo Fue bien o FASE sino. Ejemplo X==NULL.
 *  \ingroup PdsPoissonGroup
 */
int pds_poisson_get_value(PdsPoisson *X, PdsRvInteger *x);


/** \fn int pds_poisson_get_last_value(PdsPoisson *X,PdsRvInteger *x)
 *  \brief Retorna el último número pseudo aleatorio de la R.V. poissoniana,
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly.
 *  \param[in] X Variable aleatoria Poissoniana.
 *  \param[out] x Un número con una pdf, 
 *  \f$ Poisson(\lambda) \f$ \manonly Poisson(lambda)  \endmanonly .
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo Fue bien o FASE sino. Ejemplo X==NULL.
 *  \ingroup PdsPoissonGroup
 */
int pds_poisson_get_last_value(PdsPoisson *X,PdsRvInteger *x);


/** \fn void pds_poisson_free(PdsPoisson *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsPoisson.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsPoissonGroup
 */
void pds_poisson_free(PdsPoisson *X);


/** \fn void pds_poisson_destroy(PdsPoisson **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsPoisson, y la carga con NULL.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsPoissonGroup
 */
void pds_poisson_destroy(PdsPoisson **X);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif /* __PDSPOISSON_H__ */ 

