/*
 * pdsrayleigh.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdsrayleigh.h
 *  \author Fernando Pujaico Rivera
 *  \date 01-04-2011
 *  \brief Funciones que trabajan con variables aleatorias rayleighianas (pseudoaleatorias).
 *  
 *  <br><br>Estas funciones trabajan con una variable aleatoria (R.V.) X de tipo rayleighiana
 *  de media Sigma*sqrt(pi/2) (E[X]=Sigma*sqrt(pi/2)) e varianza (4-pi)/2*Sigma^2 
 *  (E[X^2-E^2[X]]=(4-pi)/2*Sigma^2), esta R.V. está generada mediante 
 *  el uso de una variable aleatoria uniformemente distribuida creada con el método
 *  congruencial mixto 2^N.
 */

#ifndef __PDSRAYLEIGH_H__
#define __PDSRAYLEIGH_H__

#include <stdio.h>
#include <pds/pdsrvglobal.h>
#include <pds/pdscongruential.h>
#include <pds/pdsvector.h>

#ifdef __cplusplus
extern "C" {
#endif 

/** \defgroup PdsRayleighGroup Funciones del módulo PdsRayleigh.
 * @{
 */



/*! \struct PdsRayleigh
 *  \brief La estructura tipo  PdsRayleigh.
 *  Esta estructura genera una variable aleatoria rayleighiana,
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  Para usar incluir pdsrv/pdsrv.h.
 *  La función de densidad de probabilidad es:
 *  \f[ f_x(x)={\frac{x}{{\sigma}^2}e^{\frac{-x^2}{2 \sigma^2}}} , x \in [0,\infty) \f]
 *  \manonly fx(x)=(x/Sigma^2)e^{-x^2/(2 Sigma^2)} x>=0  \endmanonly
 *  \ingroup PdsRayleighGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Una secuencia aleatoria congruencial*/
	PdsCongruential *X1;

	/*! El desvío padrón de la variable aletoria rayleighiana. */
	PdsRvReal Sigma;

	/*! La salida de la variable aletoria rayleighiana,
	 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
	 */
	PdsRvReal x;
}PdsRayleigh;

/** \fn PdsRayleigh *pds_rayleigh_new(PdsRvReal Sigma)
 *  \brief Crea una variable aleatoria de tipo PdsRayleigh.
 *  \param[in] Sigma Es la moda de la variable aleatoria rayleighiana.
 *  \return Un puntero a la variable aleatoria de tipo Rayleigh, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsRayleighGroup
 */
PdsRayleigh *pds_rayleigh_new(PdsRvReal Sigma);


/** \fn int pds_rayleigh_init(PdsRayleigh *X, PdsRvReal x0)
 *  \brief Inicia la variable aleatorio Rayleigh,
 *  \param[in] X Variable aleatoria Rayleigh.
 *  \param[in] x0 El valor inicial de la Variable aleatoria Rayleigh.
 *  \return TRUE si todo bien o FALSE si no. Ejemplo X=NULL o x0<0.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_init(PdsRayleigh *X, PdsRvReal x0);

/** \fn int pds_rayleigh_get_value(PdsRayleigh *X,PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio rayleighiano, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *
 *  Para generar la variable aleatoria Rayleighiana se usa una variable aleatoria
 *  uniformemente distribuida X1 entre [0,1), de la siguiente manera:
 *  \f$ X=\sigma \sqrt{-2.0 ln(1-x1)} \f$ \manonly sigma*sqrt(-2.0*ln(1-x1)) \endmanonly.
 *  \param[in] X Variable aleatoria Rayleighiana.
 *  \param[out] x Es un número con una pdf,
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_get_value(PdsRayleigh *X,PdsRvReal *x);


/** \fn int pds_rayleigh_get_last_value(PdsRayleigh *X, PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregada por la R.V. rayleighiana, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  \param[in] X Variable aleatoria Rayleighiana.
 *  \param[out] x Es un número con una pdf,
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_get_last_value(PdsRayleigh *X, PdsRvReal *x);


/** \fn int pds_rayleigh_get_vector(PdsRayleigh *X,PdsVector *x)
 *  \brief Carga un vector con números pseudo aleatorios rayleighianos, 
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *
 *  Para generar la variable aleatoria Rayleighiana se usa una variable aleatoria
 *  uniformemente distribuida X1 entre [0,1), de la siguiente manera:
 *  \f$ X=\sigma \sqrt{-2.0 ln(1-x1)} \f$ \manonly sigma*sqrt(-2.0*ln(1-x1)) \endmanonly.
 *  \param[in] X Variable aleatoria Rayleighiana.
 *  \param[out] x Es un vector con una pdf,
 *  \f$ Rayleighiana(\sigma) \f$ \manonly Rayleighiana(Sigma) \endmanonly.
 *  Si X es NULL entonces El vector no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. Ejemplo X=NULL.
 *  \ingroup PdsRayleighGroup
 */
int pds_rayleigh_get_vector(PdsRayleigh *X,PdsVector *x);


/** \fn void pds_rayleigh_free(PdsRayleigh *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsRayleigh.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsRayleighGroup
 */
void pds_rayleigh_free(PdsRayleigh *X);


/** \fn void pds_rayleigh_destroy(PdsRayleigh **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsRayleigh, y la carga con NULL.
 *  \param[in] X la variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsRayleighGroup
 */
void pds_rayleigh_destroy(PdsRayleigh **X);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif /* __PDSRAYLEIGH_H__ */ 

