/*
 * pdsuniform.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdsuniform.h
 *  \author Fernando Pujaico Rivera
 *  \date 01-04-2011
 *  \brief Funciones que trabajan con variables aleatorias uniformemente distribuidas (pseudoaleatorias).
 *  
 *  <br><br>Estas funciones trabajan con una variable aleatoria (R.V) X de tipo uniforme
 *  uniformemente distribuida entre [A,B), esta R.V. está generada mediante el método
 *  congruencial mixto (2^N).
 */

#ifndef __PDSUNIFORM_H__
#define __PDSUNIFORM_H__

#include <stdio.h>
#include <pds/pdsrvglobal.h>
#include <pds/pdscongruential.h>
#include <pds/pdsvector.h>

#ifdef __cplusplus
extern "C" {
#endif 

/** \defgroup PdsUniformGroup Funciones del módulo PdsUniform.
 * @{
 */

/*! Minima probabilidad detectable y generable, es decir una probabilidad P,
 *  P< PDS_MIN_UNIFORM_PROB será considerada como cero; dada una probabilidad
 *  cualquiera P0, la probabilidad P+P0 será considerada como P0.
 *  Esto implica que si la variable aleatoria uniforme tiene valore x. 0 <= x <=1.0,
 *  entonces el minimo valor de cambio de x es PDS_MIN_UNIFORM_PROB .
 *  \ingroup PdsUniformGroup
 */
#define PDS_MIN_UNIFORM_PROB 1.0/PDS_RAND_MAX 


/*! \struct PdsUniform
 *  \brief La estructura tipo  PdsUniform.
 *  Esta estructura genera una variable aleatoria uniformemente distribuida.
 *  Esta R.V. está generada mediante el método congruencial mixto para 2^N, N=31.
 *  Para usar, incluir pdsrv/pdsrv.h.
 *  La función de densidad de probabilidad es:
 *  \f[ f_x(x)=\frac{1}{B-A}, x \in [A,B) \f] \manonly fx(x)=1/(B-A), A<=x<B  \endmanonly
 *  \ingroup PdsUniformGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Una secuencia aleatoria generada con el método congruencial mixto.
         *  Su salida va entre 0<x0<PDS_RAND_MAX
         */
	PdsCongruential *X1;

	/*! La salida real de la variable aleatoria uniformemente distribuida  
         *  entre A<=x<B. 
         */
	PdsRvReal x;

	/*! Rango inicial izquierdo de la variable aleatoria. */
	PdsRvReal A;

	/*! Rango final derecho de la variable aleatoria. */
	PdsRvReal B;
}PdsUniform;

/** \fn PdsUniform *pds_uniform_new(PdsRvReal A,PdsRvReal B)
 *  \brief Crea una variable aleatoria de tipo PdsUniform.
 *  \param[in] A Rango inicial izquierdo de la variable aleatoria.
 *  \param[in] B Rango final derecho de la variable aleatoria.
 *  \return Un puntero a la variable aleatoria de tipo PdsUniform.
 *  La función retorna NULL en caso de error.
 *  \ingroup PdsUniformGroup
 */
PdsUniform *pds_uniform_new(PdsRvReal A,PdsRvReal B);


/** \fn int pds_uniform_init(PdsUniform *X, PdsRvReal x0)
 *  \brief Inicializa la variable pseudo aleatorio y uniforme entre [A,B).
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[in] x0 X{inicial}=x0, si esta fuera de [A,B),X{inicial}=A.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_init(PdsUniform *X, PdsRvReal x0);


/** \fn int pds_uniform_get_value(PdsUniform *X, PdsRvReal *x)
 *  \brief Retorna un número pseudo aleatorio y uniforme entre [A,B), 
 *        con PDS_RAND_MAX divisiones de resolución.
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[out] x x es cargado con un número entre [A,B). Incluye A pero no  B.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_get_value(PdsUniform *X, PdsRvReal *x);


/** \fn int pds_uniform_get_last_value(PdsUniform *X, PdsRvReal *x)
 *  \brief Retorna el último número pseudo aleatorio entregado por la R.V. uniforme entre [A,B), 
 *        con PDS_RAND_MAX divisiones de resolución.
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[out] x x es cargado con un número entre [A,B). Incluye A pero no  B.
 *  Si X es NULL entonces retorna 0.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_get_last_value(PdsUniform *X, PdsRvReal *x);


/** \fn int pds_uniform_get_vector(PdsUniform *X, PdsVector *x)
 *  \brief Retorna un vector con valores pseudo aleatorio y uniforme entre [A,B), 
 *        con PDS_RAND_MAX divisiones de resolución.
 *  \param[in] X Variable aleatoria uniformemente distribuida.
 *  \param[out] x x es cargado con números entre [A,B). Incluye A pero no  B.
 *  Si X es NULL entonces el vector no es alterado.
 *  \return TRUE si todo fue bien o FALSE si no. ejemplo X=NULL.
 *  \ingroup PdsUniformGroup
 */
int pds_uniform_get_vector(PdsUniform *X, PdsVector *x);


/** \fn void pds_uniform_free(PdsUniform *X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsUniform.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsUniformGroup
 */
void pds_uniform_free(PdsUniform *X);



/** \fn void pds_uniform_destroy(PdsUniform **X)
 *  \brief Libera la variable aleatoria de tipo puntero PdsUniform, y la carga con NULL.
 *  \param[in] X La variable aleatoria a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsUniformGroup
 */
void pds_uniform_destroy(PdsUniform **X);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif /* __PDSUNIFORM_H__ */ 

