/*
 * testpdsrv1.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file testpdsrv1.c
 *  \author Fernando Pujaico Rivera
 *  \date 18-04-2011
 *  \brief Programa para el testeo de la estadística de todas las variables aleatorias de la biblioteca.
 *  
 *  \n\n Comprueba el desempeño de las variables aleatorias y genera un archivo
 *  para ser usado en octave, para la visualización.
 *  El propósito del programa es generar un número considerable de muestras de todas
 *  las R.V. para poder así corroborar de forma experimental sus propiedades 
 *  estadísticas.
 */


#include <stdlib.h>
#include <pds/pdsrv.h>

/** \def NUMELEMENTOS
 *  Define cuantos elementos de las R.V se usarán para generar de forma 
 *  experimental la función de densidad de probabilidad.
*/
#define NUMELEMENTOS PDS_RAND_MAX/1000

int pds_octave_plot_pdf(char NombreOctaveFile[],char NombreOctaveDataFile[]);


int main(int argc, char** argv)
{
	PdsRvReal x1,x2,x3,x4,x5;
	PdsRvInteger x6;
	PdsRvNaturalD xc;
	unsigned long int i;
	PdsCongruential	*Xc=NULL;
	PdsUniform		*X1=NULL;
	PdsUniform		*X2=NULL;
	PdsGaussian		*X3=NULL;
	PdsRayleigh		*X4=NULL;
	PdsExponential		*X5=NULL;
	PdsPoisson		*X6=NULL;
	FILE *fd=NULL;

	Xc=pds_congruential_new();	if(Xc==NULL)	return EXIT_FAILURE;
	X1=pds_uniform_new(1.5,3.5);	if(X1==NULL)	return EXIT_FAILURE;
	X2=pds_uniform_new(1.5,3.5);	if(X2==NULL)	return EXIT_FAILURE;
	X3=pds_gaussian_new(0.0,1.0);	if(X3==NULL)	return EXIT_FAILURE;
	X4=pds_rayleigh_new(1.0);	if(X4==NULL)	return EXIT_FAILURE;
	X5=pds_exponential_new(1.0);	if(X5==NULL)	return EXIT_FAILURE;
	X6=pds_poisson_new(4.0);	if(X6==NULL)	return EXIT_FAILURE;

	pds_octave_plot_pdf("plot_octave_pds.m","prueba.txt");

	fd=fopen("prueba.txt","w");
	if(fd==NULL)	{printf("ERROR:fopen de datos.\n");return EXIT_FAILURE;}

	for(i=0;i<NUMELEMENTOS;i++)	
	{
		pds_congruential_get_value(Xc,&xc);
		pds_uniform_get_value(X1,&x1);
		pds_uniform_get_value(X2,&x2);
		pds_gaussian_get_value(X3,&x3);
		pds_rayleigh_get_value(X4,&x4);
		pds_exponential_get_value(X5,&x5);
		pds_poisson_get_value(X6,&x6);

		fprintf(fd,"%f\t%f\t%f\t%f\t%f\t%f\t%f\n",x1,x2,x3,x4,x5,1.0*x6,1.0*xc);
	}

	fclose(fd);

	pds_congruential_destroy(&Xc);
	pds_uniform_destroy(&X1);
	pds_uniform_destroy(&X2);
	pds_gaussian_destroy(&X3);
	pds_rayleigh_destroy(&X4);
	pds_exponential_destroy(&X5);
	pds_poisson_destroy(&X6);

	return EXIT_SUCCESS;	
}


/** \fn int pds_octave_plot_pdf(char NombreOctaveFile[],char NombreOctaveDataFile[])
 *  \brief Función para la creación de un archivo *.m de octave, para mostrar
 *         experimentalmente la función de densidad de probabilidad de las R.V.
 *  \param[in] NombreOctaveFile Nombre del archivo *.m que se creará.
 *  \param[in] NombreOctaveDataFile Nombre del archivo donde se tiene los valores 
 *             de las R.V. separadas en columnas.
 *  \return Retorna EXIT_SUCCESS si todo fue bien y EXIT_FAILURE si no.
 */
int pds_octave_plot_pdf(char NombreOctaveFile[],char NombreOctaveDataFile[])
{
	FILE *fd;

	fd=fopen(NombreOctaveFile,"w");
	if(fd==NULL)	{printf("ERROR:fopen de octave.\n");return EXIT_FAILURE;}

	fprintf(fd,"X=load(\'%s\')\';\n",NombreOctaveDataFile);	
	fprintf(fd,"intervalos=100;\n");
	fprintf(fd,"\n");
	fprintf(fd,"[Nj1 abscisax1]=hist(X(1,:),intervalos);\n");
	fprintf(fd,"Rango1=max(X(1,:))-min(X(1,:));\n");
	fprintf(fd,"figure(1);bar(abscisax1,Nj1*intervalos/(Rango1*%lu));\n",NUMELEMENTOS);
	fprintf(fd,"\n");
	fprintf(fd,"[Nj2 abscisax2]=hist(X(2,:),intervalos);\n");
	fprintf(fd,"Rango2=max(X(2,:))-min(X(2,:));\n");
	fprintf(fd,"figure(2);bar(abscisax2,Nj2*intervalos/(Rango2*%lu));\n",NUMELEMENTOS);
	fprintf(fd,"\n");
	fprintf(fd,"Ex1=mean(X(1,:))\n");
	fprintf(fd,"Ex2=mean(X(2,:))\n");
	fprintf(fd,"cov_xy=mean(cov((X(1,:)-Ex1),(X(2,:)-Ex1)))\n");	
	fprintf(fd,"\n");
	fprintf(fd,"[Nj3 abscisax3]=hist(X(3,:),intervalos);\n");
	fprintf(fd,"Rango3=max(X(3,:))-min(X(3,:));\n");
	fprintf(fd,"figure(3);bar(abscisax3,Nj3*intervalos/(Rango3*%lu));\n",NUMELEMENTOS);
	fprintf(fd,"Ex3=mean(X(3,:))\n");
	fprintf(fd,"E2x3=mean((X(3,:)-Ex3).*(X(3,:)-Ex3))\n");
	fprintf(fd,"\n");
	fprintf(fd,"[Nj4 abscisax4]=hist(X(4,:),intervalos);\n");
	fprintf(fd,"Rango4=max(X(4,:))-min(X(4,:));\n");
	fprintf(fd,"figure(4);bar(abscisax4,Nj4*intervalos/(Rango4*%lu));\n",NUMELEMENTOS);
	fprintf(fd,"Ex4=mean(X(4,:))\n");
	fprintf(fd,"E2x4=mean((X(4,:)-Ex4).*(X(4,:)-Ex4))\n");
	fprintf(fd,"\n");
	fprintf(fd,"[Nj5 abscisax5]=hist(X(5,:),intervalos);\n");
	fprintf(fd,"Rango5=max(X(5,:))-min(X(5,:));\n");
	fprintf(fd,"figure(5);bar(abscisax5,Nj5*intervalos/(Rango5*%lu));\n",NUMELEMENTOS);
	fprintf(fd,"Ex5=mean(X(5,:))\n");
	fprintf(fd,"E2x5=mean((X(5,:)-Ex5).*(X(5,:)-Ex5))\n");
	fprintf(fd,"\n");
	fprintf(fd,"[Nj6 abscisax6]=hist(X(6,:),intervalos);\n");
	fprintf(fd,"Rango6=max(X(6,:))-min(X(6,:));\n");
	fprintf(fd,"figure(6);bar(abscisax6,Nj6*1.0/(%lu));\n",NUMELEMENTOS);
	fprintf(fd,"Ex6=mean(X(6,:))\n");
	fprintf(fd,"E2x6=mean((X(6,:)-Ex6).*(X(6,:)-Ex6))\n");
	fprintf(fd,"\n");
	fprintf(fd,"\n");
	fprintf(fd,"C=min(size(X))\n");
	fprintf(fd,"[NjC abscisaxC]=hist(X(C,:),intervalos);\n");
	fprintf(fd,"RangoC=max(X(C,:))-min(X(C,:));\n");
	fprintf(fd,"figure(C);bar(abscisaxC,NjC*intervalos/(RangoC*%lu));\n",NUMELEMENTOS);
	fprintf(fd,"\n");
	fclose(fd);

	return EXIT_SUCCESS;
}

