/*
 * testpdsrv3.c
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file testpdsrv3.c
 *  \author Fernando Pujaico Rivera
 *  \date 18-04-2011
 *  \brief Programa para averiguar cuantas variables aletorias distintas es posible de generar.
 *
 *  \n\n Todas las R.V. usan un generador secuencial para generarse, usando la
 *  ecuación xn=[a*x0+c]%PDS_RAND_MAX. En esta ecuación "a" es fijo, pero "c"
 *  es distinto para cada variable aleatoria nueva creada, la pregunta es: cuantas?
 *  Respuesta: 7830 variables aleatorias distintas. según el algoritmo que usé.
 */

#include <config.h>
#include <stdlib.h>
#include <math.h>
#include <pds/pdsrv.h>


double pds_inc(	unsigned long int t3,unsigned long int t5,
			unsigned long int t7,unsigned long int t11,
			unsigned long int t13,unsigned long int t17);


void pds_next_inc(	unsigned long int *t3 ,unsigned long int *t5,
			unsigned long int *t7 ,unsigned long int *t11,
			unsigned long int *t13,unsigned long int *t17);

int main(int argc, char** argv)
{
	unsigned long int i;
	unsigned long int t3=0;
	unsigned long int t5=0;
	unsigned long int t7=0;
	unsigned long int t11=0;
	unsigned long int t13=0;
	unsigned long int t17=0;
	
	PdsRvReal C0,C1;

	i=0;
	C0=pds_inc(t3,t5,t7,t11,t13,t17);
	printf("%10lu:::%10lu:%10lu:{%3lu,%3lu,%3lu,%3lu,%3lu,%3lu}\n",i,PDS_RAND_MAX,(unsigned long int)C0,t3,t5,t7,t11,t13,t17);
	pds_next_inc(&t3,&t5,&t7,&t11,&t13,&t17);

	do
	{
		i++;
		C1=pds_inc(t3,t5,t7,t11,t13,t17);
		printf("%10lu:::%10lu:%10lu:{%3lu,%3lu,%3lu,%3lu,%3lu,%3lu}\n",i,PDS_RAND_MAX,(unsigned long int)C1,t3,t5,t7,t11,t13,t17);
		pds_next_inc(&t3,&t5,&t7,&t11,&t13,&t17);
		//getchar();

	}while(C0!=C1);

	printf("<<Numero total de variables aleatorias distintas : %lu>>\n",i);

	return EXIT_SUCCESS;
}


/** \fn double pds_inc(	unsigned long int t3,unsigned long int t5,
				unsigned long int t7,unsigned long int t11,
				unsigned long int t13,unsigned long int t17)
 *  \brief Evalúa la siguiente función f(t3,t5,t7,t11,t13,t17)=3^t3 5^t5 7^t7 11^t11 13^t13 17^t17.
 *  \f$ f(t3,t5,t7,t11,t13,t17)=3^{t3} 5^{t5} 7^{t7} 11^{t11} 13^{t13} 17^{t17} \f$
 *  \param[in] t3  Exponente de 3.
 *  \param[in] t5  Exponente de 5.
 *  \param[in] t7  Exponente de 7.
 *  \param[in] t11 Exponente de 11.
 *  \param[in] t13 Exponente de 13.
 *  \param[in] t17 Exponente de 17.
 *  \return Retorna f(t3,t5,t7,t11,t13,t17);
 */
double pds_inc(	unsigned long int t3,unsigned long int t5,
			unsigned long int t7,unsigned long int t11,
			unsigned long int t13,unsigned long int t17)
{
	return pow(3.0,t3)*pow(5.0,t5)*pow(7.0,t7)*pow(11.0,t11)*pow(13.0,t13)*pow(17.0,t17);
}

/** \fn void pds_next_inc(	unsigned long int *t3 ,unsigned long int *t5,
			unsigned long int *t7 ,unsigned long int *t11,
			unsigned long int *t13,unsigned long int *t17)
 *  \brief Evalúa el siguiente valor que tomará {t3,t5,t7,t11,t13,t17}. 
 *
 *  Evalúa el siguiente valor que tomará {t3,t5,t7,t11,t13,t17} considerando que
 *  f(t3,t5,t7,t11,t13,t17) tiene que ser menor que PDS_RAND_MAX.
 *  \param[in] t3  Exponente de 3.
 *  \param[in] t5  Exponente de 5.
 *  \param[in] t7  Exponente de 7.
 *  \param[in] t11 Exponente de 11.
 *  \param[in] t13 Exponente de 13.
 *  \param[in] t17 Exponente de 17.
 */
void pds_next_inc(	unsigned long int *t3 ,unsigned long int *t5,
			unsigned long int *t7 ,unsigned long int *t11,
			unsigned long int *t13,unsigned long int *t17)
{

	if(pds_inc((*t3)+1,*t5,*t7,*t11,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t3=(*t3)+1;}
	else
	{	*t3=1; 
		if(pds_inc(*t3,(*t5)+1,*t7,*t11,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t5=(*t5)+1;}
		else
		{	*t5=0;
			if(pds_inc(*t3,*t5,(*t7)+1,*t11,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t7=(*t7)+1;}
			else
			{	*t7=0;
				if(pds_inc(*t3,*t5,*t7,(*t11)+1,*t13,*t17)<(PDS_RAND_MAX*1.0))	{*t11=(*t11)+1;}
				else
				{	*t11=0;
					if(pds_inc(*t3,*t5,*t7,*t11,(*t13)+1,*t17)<(PDS_RAND_MAX*1.0))	{*t13=(*t13)+1;}
					else
					{	*t13=0;
						if(pds_inc(*t3,*t5,*t7,*t11,*t13,(*t17)+1)<(PDS_RAND_MAX*1.0))	{*t17=(*t17)+1;}
						else									{*t17=0;*t3=0;*t5=0;*t7=0;*t11=0;*t13=0;}
					}
				}
			}
		}		
	}

}
